angular.module('internalSearches', ['wemServices', 'wemDirectives', 'ui.bootstrap', 'i18n', 'underscore'])
    .run(['$rootScope', function ($rootScope) {
        $rootScope.getCurrentModuleResource = function (resourcePath) {
            return ManagersUtils.getCurrentModuleResource(resourcePath);
        };

        $rootScope.pagerUrl = ManagersContext.localTemplatesPath + "/dt-pager.html";
    }])

    .controller('InternalSearchesListCtrl', ['$scope', '$timeout', 'datepickerConfig', 'DTInstances', 'DTOptionsBuilder', 'DTColumnBuilder', '$compile', 'dataTableService', 'i18nService', '$location', 'internalSearchService', '_',
        function ($scope, $timeout, datepickerConfig, DTInstances, DTOptionsBuilder, DTColumnBuilder, $compile, dataTableService, i18nService, $location, internalSearchService, _) {

            var refreshPager = function () {
                $timeout(function(){
                    ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                    // need to call $scope.$digest to force variable link of the pager when reordering.
                    if (!$scope.$$phase) {
                        $scope.$digest();
                    }
                });
            };

            $(document).on('length.dt', function(event){
                refreshPager();
            });

            $scope.$watch('dtInstance', function(val) {
                if (val) {
                    refreshPager();
                }
            });

            $scope.internalSearchFrom = Date.now() + -30*24*3600*1000;
            $scope.internalSearchTo = Date.now();

            var actionsHtml = function (data, type, full, meta) {
                return "<button title=\"" + i18nService.message("serverSettings.internalSearches.goToDetails") + "\" type=\"button\" class=\"btn btn-icon command-dashboard\" ng-click=\"goToDetails(" + meta.row + ")\"><span class=\"md md-searches\"></span></button> ";
            };

            //No global controller where to put this function, it must be redefined for each app
            var rowCallback = function (nRow, aData, iDisplayIndex, iDisplayIndexFull) {
                $compile(nRow)($scope);
            };

            var init = function (from, to) {
                var promise = internalSearchService.getSearchOccurrencesByTermAndDate(from, to);
                dataTableService.setPromise(promise);
                $scope.dtOptions = dataTableService.generateDataTableOptions(
                    {defaultOrder: [[2, 'desc']], rowCallback: rowCallback}
                );

                var columns = [
                    {name: 'searchString', bundleKey: 'serverSettings.internalSearches.searchString'},
                    {name: 'nbOcc', bundleKey: 'serverSettings.internalSearches.occurrences'},
                    {name: 'importance', bundleKey: 'serverSettings.internalSearches.importance'},
                    {name: null, bundleKey: 'serverSettings.manageWem.listAction',
                        actionsHtml: actionsHtml,
                        notSortable: true,
                        options: [
                            ["width", "180px"],
                            ["contentPadding", "0"]
                        ]
                    }
                ];
                $scope.dtColumns = dataTableService.generateDataTableColumns(columns);
                promise.then(function(){
                    refreshPager();
                });
            };

            init($scope.internalSearchFrom, $scope.internalSearchTo);

            // date config
            datepickerConfig.startingDay = 1;
            datepickerConfig.minMode = "day";
            datepickerConfig.datepickerMode = "day";
            datepickerConfig.showWeeks = false;
            $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
            $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
            $scope.datePickers = {};

            $scope.openDatePopup = function ($event, selector) {
                $event.preventDefault();
                $event.stopPropagation();

                angular.forEach($scope.datePickers, function(value, key) {
                    $scope.datePickers[key] = false;
                });
                $scope.datePickers[selector] = true;
            };

            $scope.setDtInstance = function (instance) {
                $scope.dtInstance = instance;
                ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
            };

            $scope.goToDetails = function (index) {
                internalSearchService.selectSearch($scope.dtInstance.DataTable.row(index).data().searchString);
                // the redirect to search details view using a fake route parameter so that we can properly find the previous page in profiles controllers
                $location.path('/search/selected');
            };

            $scope.updateSearchesList = function () {
                init(new Date($scope.internalSearchFrom).getTime(), new Date($scope.internalSearchTo).getTime());
                $scope.dtInstance.dataTable.fnDraw();
                //refreshPager();
            };

            $scope.getMessage = function (key) {
                return i18nService.message(key);
            };
    }])

    .controller('InternalSearchesCtrl', ['$scope', '$routeParams', 'internalSearchService', 'i18nService', '$location', 'notificationService',
        function ($scope, $routeParams, internalSearchService, i18nService, $location, notificationService) {

            $scope.$on("$routeChangeError", function (evt, current, previous, rejection) {
                if (rejection == "profile_not_found") {
                    notificationService.error(i18nService.message("route.reject.profile_not_found"))
                }
            });

            $scope.searchDetail = internalSearchService.getSearchDetails();

            $scope.getTabTitle = function (key) {
                return i18nService.message(key);
            };
        }]);
